<?php

namespace Fundamento;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Plugin class.
 *
 * The main class that initiates and runs the addon.
 *
 * @since 1.0.0
 */
final class Plugin {

	/**
	 * Addon Version
	 *
	 * @since 1.0.0
	 * @var string The addon version.
	 */
	const VERSION = '1.1.2';

	/**
	 * Minimum Elementor Version
	 *
	 * @since 1.0.0
	 * @var string Minimum Elementor version required to run the addon.
	 */
	const MINIMUM_ELEMENTOR_VERSION = '3.5.0';

	/**
	 * Minimum PHP Version
	 *
	 * @since 1.0.0
	 * @var string Minimum PHP version required to run the addon.
	 */
	const MINIMUM_PHP_VERSION = '7.4';

	/**
	 * Instance
	 *
	 * @since 1.0.0
	 * @access private
	 * @static
	 * @var \Fundamento\Plugin The single instance of the class.
	 */
	private static $_instance = null;

	/**
	 * Instance
	 *
	 * Ensures only one instance of the class is loaded or can be loaded.
	 *
	 * @since 1.0.0
	 * @access public
	 * @static
	 * @return \Fundamento\Plugin An instance of the class.
	 */
	public static function instance() {

		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;

	}

	/**
	 * Constructor
	 *
	 * Perform some compatibility checks to make sure basic requirements are meet.
	 * If all compatibility checks pass, initialize the functionality.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function __construct() {

		if ( $this->is_compatible() ) {
			add_action( 'elementor/init', [ $this, 'init' ], 11 );
		}

	}

	/**
	 * Compatibility Checks
	 *
	 * Checks whether the site meets the addon requirement.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function is_compatible() {

		// Check if Elementor installed and activated
		if ( ! did_action( 'elementor/loaded' ) ) {
			add_action( 'admin_notices', [ $this, 'admin_notice_missing_main_plugin' ] );
			return false;
		}

		// Check for required Elementor version
		if ( ! version_compare( ELEMENTOR_VERSION, self::MINIMUM_ELEMENTOR_VERSION, '>=' ) ) {
			add_action( 'admin_notices', [ $this, 'admin_notice_minimum_elementor_version' ] );
			return false;
		}

		// Check for required PHP version
		if ( version_compare( PHP_VERSION, self::MINIMUM_PHP_VERSION, '<' ) ) {
			add_action( 'admin_notices', [ $this, 'admin_notice_minimum_php_version' ] );
			return false;
		}

		return true;

	}

	/**
	 * Admin notice
	 *
	 * Warning when the site doesn't have Elementor installed or activated.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function admin_notice_missing_main_plugin() {

		if ( isset( $_GET['activate'] ) ) unset( $_GET['activate'] );

		$message = sprintf(
			/* translators: 1: Plugin name 2: Elementor */
			esc_html__( '"%1$s" requires "%2$s" to be installed and activated.', 'fundamento' ),
			'<strong>' . esc_html__( 'Fundamento', 'fundamento' ) . '</strong>',
			'<strong>' . esc_html__( 'Elementor', 'fundamento' ) . '</strong>'
		);

		printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );

	}

	/**
	 * Admin notice
	 *
	 * Warning when the site doesn't have a minimum required Elementor version.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function admin_notice_minimum_elementor_version() {

		if ( isset( $_GET['activate'] ) ) unset( $_GET['activate'] );

		$message = sprintf(
			/* translators: 1: Plugin name 2: Elementor 3: Required Elementor version */
			esc_html__( '"%1$s" requires "%2$s" version %3$s or greater.', 'fundamento' ),
			'<strong>' . esc_html__( 'Fundamento ', 'fundamento' ) . '</strong>',
			'<strong>' . esc_html__( 'Elementor', 'fundamento' ) . '</strong>',
			 self::MINIMUM_ELEMENTOR_VERSION
		);

		printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );

	}

	/**
	 * Admin notice
	 *
	 * Warning when the site doesn't have a minimum required PHP version.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function admin_notice_minimum_php_version() {

		if ( isset( $_GET['activate'] ) ) unset( $_GET['activate'] );

		$message = sprintf(
			/* translators: 1: Plugin name 2: PHP 3: Required PHP version */
			esc_html__( '"%1$s" requires "%2$s" version %3$s or greater.', 'fundamento' ),
			'<strong>' . esc_html__( 'Fundamento', 'fundamento' ) . '</strong>',
			'<strong>' . esc_html__( 'PHP', 'fundamento' ) . '</strong>',
			 self::MINIMUM_PHP_VERSION
		);

		printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );

	}

	/**
	 * Admin notice
	 *
	 * Warning when the site is trying to call register_skin outside of elementor/init action
	 *
	 * @since 1.0.4
	 * @access public
	 */
	public function admin_notice_wrong_register_skin_location() {
		$message = sprintf(
			/* translators: 1: Plugin name 2: PHP 3: Required PHP version */
			esc_html__( '%1$s: register_skin() should only be called in the "elementor/init" action.', 'fundamento' ),
			'<strong>' . esc_html__( 'Fundamento', 'fundamento' ) . '</strong>',
		);

		printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );
	}

  /**
	 * Admin notice
	 *
	 * Warning when the site is trying to call register_padding outside of elementor/init action
	 *
	 * @since 1.1.0
	 * @access public
	 */
	public function admin_notice_wrong_register_padding_location() {
		$message = sprintf(
			/* translators: 1: Plugin name 2: PHP 3: Required PHP version */
			esc_html__( '%1$s: register_padding() should only be called in the "elementor/init" action.', 'fundamento' ),
			'<strong>' . esc_html__( 'Fundamento', 'fundamento' ) . '</strong>',
		);

		printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );
	}

	/**
	 * Initialize
	 *
	 * Load the addons functionality only after Elementor is initialized.
	 *
	 * Fired by `elementor/init` action hook.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function init() {
		
		$this->create_skin_controls();
		
	}

  private $padding_presets = [];

  /**
	* Register elementor widget padding preset
	*
	* @param padding $padding array containing the following "is_default", "name", "element", "padding"
	*
	* @since 1.1.0
	* @access public
	*/
  public function register_padding($padding) {

    if(current_action() !== 'elementor/init') {
      add_action( 'admin_notices', [ $this, 'admin_notice_wrong_register_padding_location' ] );
    }

    $this->padding_presets[] = $padding;

  }

	
	private $skins = [];

	/**
	* Register elementor widget skin
	*
	* @param skin $skin array containing the following "is_default", "name", "element", "css", "js", "template"
	*
	* @since 1.0.0
	* @access public
	*/
	public function register_skin($skin) {

		if(current_action() !== 'elementor/init') {
			add_action( 'admin_notices', [ $this, 'admin_notice_wrong_register_skin_location' ] );
		}

		$this->skins[] = $skin;

	}

	function create_skin_controls() {

		require_once(__DIR__ . '/controls/skin-control.php');
		require_once(__DIR__ . '/controls/padding-control.php');

		$skins = $this->skins;
    $padding_presets = $this->padding_presets;

		$skin_groups = [];
    $padding_groups = [];

		foreach($this->skins as $skin) {
			$skin_groups[$skin['element']][] = $skin;
		}
		foreach($this->padding_presets as $padding) {
			$padding_groups[$padding['element']][] = $padding;
		}
    

		foreach($skin_groups as $element => $skins) {
			new Skin_Control( $skins );
		}
		foreach($padding_groups as $element => $padding_presets) {
			new Padding_Control( $padding_presets );
		}
	}

}
